!-*- mode: F90; mode: font-lock; column-number-mode: true -*-!

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%!
!                                                                 !
! BANDS                                                           !
!                                                                 !
! Uses:   a) produce a kpoint path from a list of special points  !
!         b) produce a bandstructure plot from a nscf calculation !
!                                                                 !
! Author: Jonathan Yates 2007 email: jry20@cam.ac.uk              !
! Includes some routines taken from wannier90 www.wannier.org     !
!                                                                 !
! Copyright (C) J.R.Yates 2007                                    !
!                                                                 !
! This file is distributed under the terms of the GNU             !
! General Public License. See the file `LICENSE' in               !
! the root directory of the present distribution, or              !
! http://www.gnu.org/copyleft/gpl.txt .                           !
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%!


module constants
  implicit none
  integer, parameter :: dp = kind(1.0d0)
  real(kind=dp), parameter, public    :: pi=3.141592653589793238462643383279_dp
  real(kind=dp), parameter, public    :: twopi = 2*pi
  real(kind=dp), parameter, public :: bohr = 0.5291772108_dp
  ! max_bands is hardwired.
  integer, parameter, public  :: max_bands=1000
  integer, parameter, public :: maxlen = 120
end module constants

module data
  use constants
  implicit none
  real(kind=dp),   allocatable  :: kpath_len(:)
  integer,         allocatable  :: kpath_pts(:)
  real(kind=dp),   allocatable  :: xval(:)
  real(kind=dp),   allocatable  :: plot_kpoint(:,:)
  character(len=6),allocatable  :: glabel(:)
  character(len=3),allocatable  :: bands_label(:)
  real(kind=dp),   allocatable  :: bands_spec_points(:,:)
  real (kind=dp),  allocatable  :: bands(:,:)
  logical                       :: spin_polarised
end module data


program bands_prog
  use constants
  use data
  implicit none

  integer :: ierr = 0
  integer :: ios = 0

  character (len=100) :: dummy,dummy2
  integer :: pos,length,start,kpt,i,counter
  real(kind=dp)        :: vec(3),emin,emax
  integer              :: loop_spts,total_pts,loop_i,nkp
  integer              :: num_paths,num_spts
  integer              :: bndunit,gnuunit
  logical              :: write_path
  character(len=50)    :: seedname
  integer :: bands_num_spec_points,  bands_num_points, num_bands
  real(kind=dp) :: recip_metric(3,3)
  real(kind=dp) :: recip_lattice(3,3)
  real(kind=dp) :: real_lattice(3,3)
  real(kind=dp) :: cell_volume


  spin_polarised = .false.

  call io_get_seedname

  call internal_read_data

  allocate(kpath_len(bands_num_spec_points/2),stat=ierr)
  if (ierr/=0) call io_error('Error in allocating kpath_len in bands')
  allocate(kpath_pts(bands_num_spec_points/2),stat=ierr)
  if (ierr/=0) call io_error('Error in allocating kpath_len in bands')

  call internal_compute_path

  allocate(bands(max_bands,2*total_pts),stat=ierr)
  if (ierr/=0) call io_error('Error in allocating bands in bands')
  bands=0.0_dp

  if(write_path) then
     !
     ! Write out the kpoints in the path
     !
     bndunit=io_file_unit()
     open(bndunit,file=trim(seedname)//'_band.kpt',form='formatted')
     write(bndunit,*) total_pts
     do loop_spts=1,total_pts
        write(bndunit,'(3f12.6,3x,f3.1)') (plot_kpoint(loop_i,loop_spts),loop_i=1,3),1.0_dp
     end do
     close(bndunit)
     !
  else
     !
     ! Parse the PWSCF output file and write plotting files
     ! 
     call internal_parse_pwscf
     call internal_write_gnuplot
     call internal_write_xmgrace
     !
  end if


  if(allocated(kpath_len)) deallocate( kpath_len)
  if(allocated(kpath_pts))deallocate( kpath_pts)
  if(allocated(xval))deallocate( xval)
  if(allocated(plot_kpoint))deallocate( plot_kpoint)
  if(allocated(glabel))deallocate( glabel)
  if(allocated(bands_label))deallocate( bands_label)
  if(allocated(bands_spec_points))deallocate( bands_spec_points)
  if(allocated(bands))deallocate( bands)



contains


  subroutine internal_compute_path
    !
    ! Work out how many points in the total path 
    ! and the positions of the special points
    !
    use constants
    implicit none



    num_paths=bands_num_spec_points/2
    num_spts=num_paths+1
    do loop_spts=1,num_paths
       vec=bands_spec_points(:,2*loop_spts)-bands_spec_points(:,2*loop_spts-1)
       kpath_len(loop_spts)=sqrt(dot_product(vec,(matmul(recip_metric,vec))))
       if(loop_spts==1) then
          kpath_pts(loop_spts)=bands_num_points
       else
          kpath_pts(loop_spts)=nint(real(bands_num_points,dp)*kpath_len(loop_spts)/kpath_len(1))
       end if
    end do
    total_pts=sum(kpath_pts)+1
    allocate(plot_kpoint(3,total_pts),stat=ierr)
    if (ierr/=0) call io_error('Error in allocating plot_kpoint in internal_compute_path')
    allocate(xval(total_pts),stat=ierr)
    if (ierr/=0) call io_error('Error in allocating xval in internal_compute_path')
    allocate(glabel(num_spts),stat=ierr)
    if (ierr/=0) call io_error('Error in allocating num_spts in internal_compute_path')
    !
    ! Find the position of each kpoint in the path
    !
    counter=0
    do loop_spts=1,num_paths
       do loop_i=1,kpath_pts(loop_spts)
          counter=counter+1
          if(counter==1) then
             xval(counter)=0.0_dp
          else
             xval(counter)=xval(counter-1)+kpath_len(loop_spts)/real(kpath_pts(loop_spts),dp)
          endif
          plot_kpoint(:,counter)=bands_spec_points(:,2*loop_spts-1)+ &
               (bands_spec_points(:,2*loop_spts)-bands_spec_points(:,2*loop_spts-1))* &
               (real(loop_i-1,dp)/real(kpath_pts(loop_spts),dp))
       end do
    end do
    xval(total_pts)=sum(kpath_len)
    plot_kpoint(:,total_pts)=bands_spec_points(:,bands_num_spec_points)


    return

  end subroutine internal_compute_path


  !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  subroutine internal_parse_pwscf
  !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    !
    ! Get the bands from the pwscf output file
    !
    use constants
    implicit none


    integer :: pwscf_unit
    pwscf_unit=io_file_unit()

    open (pwscf_unit, file=trim(seedname)//'.pwscf',form='formatted',status='old',err=101)


    dummy=' '
    counter=0
    kpt=0
    do
       read(pwscf_unit,'(a)', iostat=ios) dummy
       ! check if kpoint
       if(index(dummy,'band energies')>0) then
          read(pwscf_unit,'(a)', iostat=ios) dummy
          kpt=kpt+1
          if(kpt>2*total_pts) call io_error('More kpoints in PWSCF output than in input path')
          start =0
          do ! loop over lines
             read(pwscf_unit,'(a)', iostat=ios) dummy
             if(len(trim(dummy))==0) exit
             length=1
             dummy2=adjustl(dummy)
             do 
                pos=index(dummy2,' ')
                dummy2=dummy2(pos+1:)
                dummy2=adjustl(dummy2)
                if(len_trim(dummy2)>0) then
                   length=length+1
                else
                   exit
                endif
             end do
             read(dummy,*,err=230,end=230) (bands(i,kpt),i=start+1,start+length)
             start=start+length
          end do
       end if

       if (ios .ne. 0) exit
       counter=counter+1
    end do
    num_bands=start



    if(kpt==total_pts) then
       spin_polarised=.false.
       write(*,'(1x,a,i0,a,i0,a)') 'I found ',kpt,' kpoints and ',start, ' bands'
    elseif(kpt==2*total_pts) then
       spin_polarised=.true.
       write(*,'(1x,a,i0,a,i0,a)') 'I found ',kpt/2,' kpoints, ',start, ' bands and 2 spin channels'
    else
       write(*,'(a)') ' ** WARNING **'
       write(*,'(1x,a,i0,a,i0,a)') 'I found ',kpt,' kpoints and ',start, ' bands'
       write(*,'(a,i0,a)') 'but only ',total_pts,' are defined in the input kpath'
       call io_error('Error: Problem with kpoints')
    endif





    close(pwscf_unit)

    return

230 print*, 'error'
101 call io_error('Error: Problem opening pwscf file '//trim(seedname)//'.pwscf')


  end subroutine internal_parse_pwscf


  !==================================================================!
  subroutine io_get_seedname (  )
    !==================================================================!
    !                                                                  !
    ! Get the seedname from the commandline                            !
    ! Note iargc and getarg are not standard                           !
    ! Some platforms require them to be external or provide            !
    ! equivalent routines. Not a problem in f2003!                     !
    ! (c) Wannier90 developers                                         !
    !===================================================================


#ifdef NAG
    USE F90_UNIX_ENV, ONLY : IARGC,GETARG
#endif

    implicit none

    integer :: num_arg
#ifndef NAG
    integer :: iargc
#endif
    character(len=50) :: ctemp

    write_path=.false.

    num_arg=iargc()
    if (num_arg==0) then
       seedname='bands'
    elseif (num_arg==1) then
       call getarg(1,seedname)
       if( index(seedname,'-pp')>0 ) then
          write_path=.true.
          seedname='bands'
       end if
    else
       call getarg(1,seedname)
       if( index(seedname,'-pp')>0 ) then
          write_path=.true.
          call getarg(2,seedname)
       else
          call getarg(2,ctemp)
          if( index(ctemp,'-pp')>0 ) write_path=.true.
       end if

    end if

  end subroutine io_get_seedname


  !==================================================================!
  function io_file_unit()
    !==================================================================!
    !                                                                  !
    ! Returns an unused unit number                                    !
    !  so we can open a file on that unit                              !
    ! (c) Wannier90 developers                                         !
    !                                                                  !
    !===================================================================
    implicit none

    integer :: io_file_unit,unit
    logical :: file_open

    unit = 9
    file_open = .true.
    do while ( file_open )
       unit = unit + 1
       inquire( unit, OPENED = file_open )
    end do

    io_file_unit = unit


    return
  end function io_file_unit



  !===================================================================
  subroutine utility_compute_metric
    !==================================================================!
    !                                                                  !
    !  Calculate the real and reciprical space metrics                 !
    ! (c) Wannier90 developers                                         !
    !                                                                  !
    !===================================================================

    use constants,  only : dp
    implicit none

    integer :: i,j,l

    recip_metric=0.0_dp

    do j=1,3
       do i=1,j
          do l=1,3
             recip_metric(i,j)=recip_metric(i,j)+recip_lattice(i,l)*recip_lattice(j,l)
          enddo
          if(i.lt.j) then
             recip_metric(j,i)=recip_metric(i,j)
          endif
       enddo
    enddo

  end subroutine utility_compute_metric

  !===================================================================
  subroutine utility_recip_lattice
    !==================================================================!
    !                                                                  !
    !  Calculates the reciprical lattice vectors and the cell volume   !
    ! (c) Wannier90 developers                                         !
    !                                                                  !
    !===================================================================

    use constants,  only : dp,twopi

    implicit none

    recip_lattice(1,1)=real_lattice(2,2)*real_lattice(3,3)-real_lattice(3,2)*real_lattice(2,3)
    recip_lattice(1,2)=real_lattice(2,3)*real_lattice(3,1)-real_lattice(3,3)*real_lattice(2,1)
    recip_lattice(1,3)=real_lattice(2,1)*real_lattice(3,2)-real_lattice(3,1)*real_lattice(2,2)
    recip_lattice(2,1)=real_lattice(3,2)*real_lattice(1,3)-real_lattice(1,2)*real_lattice(3,3)
    recip_lattice(2,2)=real_lattice(3,3)*real_lattice(1,1)-real_lattice(1,3)*real_lattice(3,1)
    recip_lattice(2,3)=real_lattice(3,1)*real_lattice(1,2)-real_lattice(1,1)*real_lattice(3,2)
    recip_lattice(3,1)=real_lattice(1,2)*real_lattice(2,3)-real_lattice(2,2)*real_lattice(1,3)
    recip_lattice(3,2)=real_lattice(1,3)*real_lattice(2,1)-real_lattice(2,3)*real_lattice(1,1)
    recip_lattice(3,3)=real_lattice(1,1)*real_lattice(2,2)-real_lattice(2,1)*real_lattice(1,2)

    cell_volume=real_lattice(1,1)*recip_lattice(1,1) + &
         real_lattice(1,2)*recip_lattice(1,2) + &
         real_lattice(1,3)*recip_lattice(1,3)


    if( abs(cell_volume) < 0.1e-4_dp) then
       call io_error(' Found almost zero Volume in utility_recip_lattice')
    end if

    recip_lattice=twopi*recip_lattice/cell_volume
    cell_volume=abs(cell_volume)

    return

  end subroutine utility_recip_lattice


  subroutine internal_read_data

    use constants
    implicit none

    integer           :: in_unit,tot_num_lines,ierr,line_counter,loop,in1,in2,num_lines
    integer :: cnt
    real(kind=dp) :: real_lattice_tmp(3,3)
    character(len=maxlen) :: dummy
    character(len=maxlen), allocatable :: in_data(:)

    in_unit=io_file_unit( )
    open (in_unit, file=trim(seedname)//'.inp',form='formatted',status='old',err=101)

    num_lines=0;tot_num_lines=0
    do
       read(in_unit, '(a)', iostat = ierr, err= 200, end =210 ) dummy
       dummy=adjustl(dummy)
       tot_num_lines=tot_num_lines+1
       if( .not.dummy(1:1)=='!'  .and. .not. dummy(1:1)=='#' ) then
          if(len(trim(dummy)) > 0 ) num_lines=num_lines+1
       endif

    end do

101 call io_error('Error: Problem opening input file '//trim(seedname)//'.inp')
200 call io_error('Error: Problem reading input file '//trim(seedname)//'.inp')
210 continue
    rewind(in_unit)


    allocate(in_data(num_lines),stat=ierr)
    if (ierr/=0) call io_error('Error allocating in_data in param_in_file')

    line_counter=0
    do loop=1,tot_num_lines
       read(in_unit, '(a)', iostat = ierr, err= 200 ) dummy
       dummy=adjustl(dummy)
       if( dummy(1:1)=='!' .or.  dummy(1:1)=='#' ) cycle
       if(len(trim(dummy)) == 0 ) cycle
       line_counter=line_counter+1
       in1=index(dummy,'!')
       in2=index(dummy,'#')
       if(in1==0 .and. in2==0)  in_data(line_counter)=dummy
       if(in1==0 .and. in2>0 )  in_data(line_counter)=dummy(:in2-1)
       if(in2==0 .and. in1>0 )  in_data(line_counter)=dummy(:in1-1)
       if(in2>0 .and. in1>0 )   in_data(line_counter)=dummy(:min(in1,in2)-1)
    end do

    close(in_unit)

    cnt=0
    do loop=1,3
       cnt=cnt+1
       dummy=in_data(cnt)
       read(dummy,*,err=240,end=240) (real_lattice_tmp(i,loop),i=1,3)
    end do

    real_lattice=transpose(real_lattice_tmp)
    real_lattice=bohr*real_lattice ! convert to Ang

    call utility_recip_lattice
    call utility_compute_metric
    cnt=cnt+1
    dummy=in_data(cnt)
    read(dummy,*,err=240,end=240) bands_num_points

    bands_num_spec_points=(num_lines-4)*2

    allocate(bands_spec_points(3,bands_num_spec_points))
    allocate(bands_label(bands_num_spec_points))


    do loop=1,num_lines-4
       cnt=cnt+1
       dummy=in_data(cnt)
       read(dummy,*,err=240,end=240) bands_label(2*loop-1),(bands_spec_points(i,2*loop-1),i=1,3)&
            ,bands_label(2*loop),(bands_spec_points(i,2*loop),i=1,3)
    end do

    return

240 call io_error('Error: Problem reading block keyword ')


  end subroutine internal_read_data



  !==================================================================!
  subroutine io_error ( error_msg )
    !==================================================================!
    !                                                                  !
    ! Aborts giving error message                                      !
    ! (c) Wannier90 developers                                         !
    !                                                                  !
    !===================================================================

    implicit none
    character(len=*), intent(in) :: error_msg

    write(*,*)  'Exiting.......'
    write(*, '(1x,a)') trim(error_msg)


    stop "error: examine the output/error file for details"

  end subroutine io_error

  !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  subroutine internal_write_gnuplot
  !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    !
    ! Write out the bands in gnuplot format
    !
    implicit none

    ! write band structure

    emin=minval(bands)-1.0_dp
    emax=maxval(bands)+1.0_dp
    bndunit=io_file_unit()
    open(bndunit,file=trim(seedname)//'_band.dat',form='formatted')
    gnuunit=io_file_unit()
    open(gnuunit,file=trim(seedname)//'_band.gnu',form='formatted')
    !
    ! Gnuplot format
    !
    do i=1,num_bands
       do nkp=1,total_pts
          if(.not.spin_polarised) then
             write(bndunit,'(2E16.8)') xval(nkp),bands(i,nkp)
          else
             write(bndunit,'(3E16.8)') xval(nkp),bands(i,nkp),bands(i,nkp+total_pts)
          endif
       enddo
       write(bndunit,*) ' '
    enddo
    close(bndunit)
    ! Axis labels
    glabel(1)=' '//bands_label(1)//' '
    do i=2,num_paths
       if(bands_label(2*(i-1))/=bands_label(2*(i-1)+1)) then
          glabel(i)=bands_label(2*(i-1))//'/'//bands_label(2*(i-1)+1)
       else
          glabel(i)=' '//bands_label(2*(i-1))//' '
       end if
    end do
    glabel(num_spts)=' '//bands_label(bands_num_spec_points)//' '
    ! gnu file
    write(gnuunit,701) xval(total_pts),emin,emax
    do i = 1, num_paths-1
       write(gnuunit,705) sum(kpath_len(1:i)),emin,sum(kpath_len(1:i)),emax
    enddo
    write(gnuunit,702, advance="no") glabel(1),0.0_dp,(glabel(i+1),sum(kpath_len(1:i)), &
         i=1,bands_num_spec_points/2-1)
    write(gnuunit,703) glabel(1+bands_num_spec_points/2),sum(kpath_len(:))
    if(.not.spin_polarised) then
       write(gnuunit,*) 'plot ','"'//trim(seedname)//'_band.dat','"'
    else
       write(gnuunit,*) 'plot ','"'//trim(seedname)//'_band.dat','" u 1:2,','"'//trim(seedname)//'_band.dat','" u 1:3 lt 3'
    endif
    close(gnuunit)

    return


701 format('set data style lines',/,'set nokey',/, 'set xrange [0:',F8.5,']',/,'set yrange [',F9.5,' :',F9.5,']')
702 format('set xtics (',:20('"',A3,'" ',F8.5,','))
703 format(A3,'" ',F8.5,')')
705 format('set arrow from ',F8.5,',',F10.5,' to ',F8.5,',',F10.5, ' nohead')


  end subroutine internal_write_gnuplot

  !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  subroutine internal_write_xmgrace
  !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    !
    ! Write out the bands in xmgrace format
    !
    implicit none

    ! write band structure

    emin=minval(bands)-1.0_dp
    emax=maxval(bands)+1.0_dp
    ! Axis labels
    glabel(1)=' '//bands_label(1)//' '
    do i=2,num_paths
       if(bands_label(2*(i-1))/=bands_label(2*(i-1)+1)) then
          glabel(i)=bands_label(2*(i-1))//'/'//bands_label(2*(i-1)+1)
       else
          glabel(i)=' '//bands_label(2*(i-1))//' '
       end if
    end do
    glabel(num_spts)=' '//bands_label(bands_num_spec_points)//' '

    gnuunit=io_file_unit()
    open(gnuunit,file=trim(seedname)//'_band.agr',form='formatted')
    !
    ! Xmgrace format
    !
    write(gnuunit,'(a)') '# Grace project file                      '
    write(gnuunit,'(a)') '#                                         '
    write(gnuunit,'(a)') '@version 50113                            '
    write(gnuunit,'(a)') '@page size 792, 612                       '
    write(gnuunit,'(a)') '@page scroll 5%                           '
    write(gnuunit,'(a)') '@page inout 5%                            '
    write(gnuunit,'(a)') '@link page off                            '
    write(gnuunit,'(a)') '@timestamp def "Mon Jul  2 23:07:11 2007"' 
    write(gnuunit,'(a)') '@with g0'                                  
    write(gnuunit,'(a)') '@    world xmin 0.00'
    write(gnuunit,'(a,f10.5)') '@    world xmax ',xval(total_pts)
    write(gnuunit,'(a,f10.5)') '@    world ymin ',emin
    write(gnuunit,'(a,f10.5)') '@    world ymax ',emax
    write(gnuunit,'(a)') '@default linewidth 1.5'
    write(gnuunit,'(a)') '@    xaxis  tick on'
    write(gnuunit,'(a)') '@    xaxis  tick major 1'
    write(gnuunit,'(a)') '@    xaxis  tick major color 1'
    write(gnuunit,'(a)') '@    xaxis  tick major linestyle 3'
    write(gnuunit,'(a)') '@    xaxis  tick major grid on'
    write(gnuunit,'(a)') '@    xaxis  tick spec type both'
    write(gnuunit,'(a,i0)') '@    xaxis  tick spec ',1+bands_num_spec_points/2
    write(gnuunit,'(a)') '@    xaxis  tick major 0, 0'
    do i=1,bands_num_spec_points/2
       write(gnuunit,'(a,i0,a,a)') '@    xaxis  ticklabel ',i-1,',', '"'//trim(adjustl(glabel(i)))//'"'
       write(gnuunit,'(a,i0,a,f10.5)') '@    xaxis  tick major ',i,' , ',sum(kpath_len(1:i))
    end do
    write(gnuunit,'(a,i0,a)') '@    xaxis  ticklabel ',bands_num_spec_points/2 &
         ,',"'//trim(adjustl(glabel(1+bands_num_spec_points/2)))//'"'
    write(gnuunit,'(a)') '@    xaxis  ticklabel char size 1.500000'
    write(gnuunit,'(a)') '@    yaxis  tick major 10'
    write(gnuunit,'(a)') '@    yaxis  label "Band Energy (eV)"'
    write(gnuunit,'(a)') '@    yaxis  label char size 1.500000'
    write(gnuunit,'(a)') '@    yaxis  ticklabel char size 1.500000'
    if(.not.spin_polarised) then
       do i=1,num_bands
          write(gnuunit,'(a,i0,a)') '@    s',i-1,' line color 1'
       end do
    else
       do i=1,num_bands
          write(gnuunit,'(a,i0,a)') '@    s',i-1,' line color 2'
       end do
       do i=1,num_bands
          write(gnuunit,'(a,i0,a)') '@    s',i-1+num_bands,' line color 4'
       end do
    end if
    do i=1,num_bands
       write(gnuunit,'(a,i0)') '@target G0.S',i-1
       write(gnuunit,'(a)') '@type xy'
       do nkp=1,total_pts
          write(gnuunit,'(2E16.8)') xval(nkp),bands(i,nkp)
       end do
       write(gnuunit,'(a,i0)') '&'
    end do
    if(spin_polarised) then
       do i=1,num_bands
          write(gnuunit,'(a,i0)') '@target G0.S',i-1+num_bands
          write(gnuunit,'(a)') '@type xy'
          do nkp=1,total_pts
             write(gnuunit,'(2E16.8)') xval(nkp),bands(i,nkp+total_pts)
          end do
          write(gnuunit,'(a,i0)') '&'
       end do
    end if



  end subroutine internal_write_xmgrace



end program bands_prog

